@@ -70,7 +70,7 @@ from PyQt5.QtWidgets \
 from PyQt5.QtGui \
     import (QIntValidator, QImageWriter, QStandardItemModel, QStandardItem,
             QValidator, QWheelEvent)
-
+from PyQt5.QtSvg import QSvgWidget
 from PyQt5.QtPrintSupport import (QPrintPreviewDialog, QPrintPreviewWidget,
                                   QPrinter)
 
@@ -82,18 +82,21 @@ except ImportError:
 
 try:
     from pyspread.actions import ChartDialogActions
-    from pyspread.toolbar import ChartTemplatesToolBar
+    from pyspread.toolbar import ChartTemplatesToolBar, RChartTemplatesToolBar
     from pyspread.widgets import HelpBrowser, TypeMenuComboBox
     from pyspread.lib.csv import sniff, csv_reader, get_header, convert
     from pyspread.lib.spelltextedit import SpellTextEdit
-    from pyspread.settings import TUTORIAL_PATH, MANUAL_PATH, MPL_TEMPLATE_PATH
+    from pyspread.settings import (TUTORIAL_PATH, MANUAL_PATH,
+                                   MPL_TEMPLATE_PATH, RPY2_TEMPLATE_PATH,
+                                   PLOT9_TEMPLATE_PATH)
 except ImportError:
     from actions import ChartDialogActions
-    from toolbar import ChartTemplatesToolBar
+    from toolbar import ChartTemplatesToolBar, RChartTemplatesToolBar
     from widgets import HelpBrowser, TypeMenuComboBox
     from lib.csv import sniff, csv_reader, get_header, convert
     from lib.spelltextedit import SpellTextEdit
-    from settings import TUTORIAL_PATH, MANUAL_PATH, MPL_TEMPLATE_PATH
+    from settings import (TUTORIAL_PATH, MANUAL_PATH, MPL_TEMPLATE_PATH,
+                          RPY2_TEMPLATE_PATH, PLOT9_TEMPLATE_PATH)
 
 
 class DiscardChangesDialog:
@@ -931,7 +934,7 @@ class ChartDialog(QDialog):
     """The chart dialog"""
 
     def __init__(self, parent: QWidget, key: Tuple[int, int, int],
-                 size: Tuple[int, int] = (800, 600)):
+                 size: Tuple[int, int] = (1000, 700)):
         """
         :param parent: Parent window
         :param key: Target cell for chart
@@ -949,6 +952,7 @@ class ChartDialog(QDialog):
         self.actions = ChartDialogActions(self)
 
         self.chart_templates_toolbar = ChartTemplatesToolBar(self)
+        self.rchart_templates_toolbar = RChartTemplatesToolBar(self)
 
         self.setWindowTitle(f"Chart dialog for cell {key}")
 
@@ -963,11 +967,18 @@ class ChartDialog(QDialog):
         """Event handler for pressing a template toolbar button"""
 
         chart_template_name = self.sender().data()
-        chart_template_path = MPL_TEMPLATE_PATH / chart_template_name
-        try:
-            with open(chart_template_path, encoding='utf8') as template_file:
-                chart_template_code = template_file.read()
-        except OSError:
+        chart_template_code = None
+
+        tpl_paths = MPL_TEMPLATE_PATH, RPY2_TEMPLATE_PATH, PLOT9_TEMPLATE_PATH
+        for tpl_path in tpl_paths:
+            full_tpl_path = tpl_path / chart_template_name
+            try:
+                with open(full_tpl_path, encoding='utf8') as template_file:
+                    chart_template_code = template_file.read()
+            except OSError:
+                pass
+
+        if chart_template_code is None:
             return
 
         self.editor.insertPlainText(chart_template_code)
@@ -995,7 +1006,11 @@ class ChartDialog(QDialog):
 
         # Layout
         layout = QVBoxLayout(self)
-        layout.addWidget(self.chart_templates_toolbar)
+
+        toolbar_layout = QHBoxLayout()
+        toolbar_layout.addWidget(self.chart_templates_toolbar)
+        toolbar_layout.addWidget(self.rchart_templates_toolbar)
+        layout.addLayout(toolbar_layout)
 
         layout.addWidget(self.splitter)
         layout.addWidget(buttonbox)
@@ -1023,6 +1038,18 @@ class ChartDialog(QDialog):
                 canvas.draw()
             except Exception:
                 pass
+        elif isinstance(figure, bytes) or isinstance(figure, str):
+            with redirect_stdout(filelike):
+                if isinstance(figure, str):
+                    figure = bytearray(figure, encoding='utf-8')
+                svg_widget = QSvgWidget()
+                self.splitter.replaceWidget(1, svg_widget)
+                svg_widget.renderer().load(figure)
+            stdout_str = filelike.getvalue()
+            if stdout_str:
+                stdout_str += "\n \n"
+                msg = stdout_str + f"Error:\n{figure}"
+                self.message.setText(msg)
         else:
             if isinstance(figure, Exception):
                 msg = stdout_str + f"Error:\n{figure}"
@@ -1438,7 +1465,7 @@ class CsvImportDialog(QDialog):
         return button_box
 
     def _sniff_dialect(self):
-        """"""
+        """Sniff the dialect of self.filepath`"""
 
         try:
             return sniff(self.filepath, self.sniff_size, self.csv_encoding)
