@@ -64,7 +64,12 @@ except ImportError:
 try:
     from pyspread import commands
     from pyspread.dialogs import DiscardDataDialog
-    from pyspread.grid_renderer import painter_save, CellRenderer, QColorCache
+    from pyspread.grid_renderer import (painter_save, CellRenderer,
+                                        QColorCache, BorderWidthBottomCache,
+                                        BorderWidthRightCache,
+                                        EdgeBordersCache,
+                                        BorderColorRightCache,
+                                        BorderColorBottomCache)
     from pyspread.model.model import (CodeArray, CellAttribute,
                                       DefaultCellAttributeDict)
     from pyspread.lib.attrdict import AttrDict
@@ -79,7 +84,10 @@ try:
 except ImportError:
     import commands
     from dialogs import DiscardDataDialog
-    from grid_renderer import painter_save, CellRenderer, QColorCache
+    from grid_renderer import (painter_save, CellRenderer, QColorCache,
+                               BorderWidthBottomCache, BorderWidthRightCache,
+                               EdgeBordersCache, BorderColorRightCache,
+                               BorderColorBottomCache)
     from model.model import CodeArray, CellAttribute, DefaultCellAttributeDict
     from lib.attrdict import AttrDict
     from lib.selection import Selection
@@ -105,8 +113,6 @@ class Grid(QTableView):
 
         self.main_window = main_window
 
-        self.qcolor_cache = QColorCache(self)
-
         shape = main_window.settings.shape
 
         if model is None:
@@ -116,6 +122,13 @@ class Grid(QTableView):
 
         self.setModel(self.model)
 
+        self.qcolor_cache = QColorCache(self)
+        self.borderwidth_bottom_cache = BorderWidthBottomCache(self)
+        self.borderwidth_right_cache = BorderWidthRightCache(self)
+        self.edge_borders_cache = EdgeBordersCache()
+        self.border_color_bottom_cache = BorderColorBottomCache(self)
+        self.border_color_right_cache = BorderColorRightCache(self)
+
         self.table_choice = main_window.table_choice
 
         self.widget_indices = []  # Store each index with an indexWidget here
@@ -550,14 +563,21 @@ class Grid(QTableView):
             top, left, bottom, right = merge_area
             merge_sel = Selection([(top, left)], [(bottom, right)], [], [], [])
 
-        return not(self.selection.single_cell_selected()
-                   or merge_sel.get_bbox() == self.selection.get_bbox())
+        return not (self.selection.single_cell_selected()
+                    or merge_sel.get_bbox() == self.selection.get_bbox())
 
     # Event handlers
 
     def on_data_changed(self):
         """Event handler for data changes"""
 
+        self.qcolor_cache.clear()
+        self.borderwidth_bottom_cache.clear()
+        self.borderwidth_right_cache.clear()
+        self.edge_borders_cache.clear()
+        self.border_color_bottom_cache.clear()
+        self.border_color_right_cache.clear()
+
         if not self.main_window.settings.changed_since_save:
             self.main_window.settings.changed_since_save = True
             main_window_title = "* " + self.main_window.windowTitle()
@@ -598,29 +618,33 @@ class Grid(QTableView):
             return
 
         try:
-            bbox = self.selection.get_bbox()
+            selection = self.selection
+            code_array = self.model.code_array
+            single_cell_selected = selection.single_cell_selected()
         except AttributeError:
             return
 
-        if bbox[0] != bbox[1]:
-            selected_cell_gen = self.selection.cell_generator(self.model.shape,
-                                                              self.table)
-            cell_list = list(selected_cell_gen)
-            msg = f"Selection: {len(cell_list)} cells"
-
-            res_gen = (self.model.code_array[key] for key in cell_list)
-            sum_list = [res for res in res_gen if res is not None]
-            msg_tpl = "     " + "     ".join(["Σ={}", "max={}", "min={}"])
-            if sum_list:
-                try:
-                    msg += msg_tpl.format(sum(sum_list),
-                                          max(sum_list), min(sum_list))
-                except Exception:
-                    pass
-
-            self.main_window.statusBar().showMessage(msg)
-        else:
+        if not selection or single_cell_selected:
             self.main_window.statusBar().clearMessage()
+            return
+
+        selected_cell_list = list(selection.cell_generator(self.model.shape,
+                                                           self.table))
+
+        res_gen = (code_array[key] for key in selected_cell_list
+                   if code_array(key))
+        sum_list = list(filter(None, res_gen))
+
+        msg_tpl = "     " + "     ".join(["Σ={}", "max={}", "min={}"])
+        msg = f"Selection: {len(selected_cell_list)} cells"
+        if sum_list:
+            try:
+                msg += msg_tpl.format(sum(sum_list), max(sum_list),
+                                      min(sum_list))
+            except Exception:
+                pass
+
+        self.main_window.statusBar().showMessage(msg)
 
     def on_row_resized(self, row: int, old_height: float, new_height: float):
         """Row resized event handler
@@ -721,9 +745,6 @@ class Grid(QTableView):
             key = literal_eval(repr_key)
             self._refresh_frozen_cell(key)
 
-        cell_attributes._attr_cache.clear()
-        cell_attributes._table_cache.clear()
-        self.model.code_array.result_cache.clear()
         self.model.dataChanged.emit(QModelIndex(), QModelIndex())
 
     def refresh_selected_frozen_cells(self):
@@ -2302,6 +2323,10 @@ class GridCellDelegate(QStyledItemDelegate):
         key = index.row(), index.column(), self.grid.table
         figure = self.code_array[key]
 
+        if isinstance(figure, bytes) or isinstance(figure, str):
+            # We try rendering the content as SVG
+            return self._render_svg(painter, rect, index, figure)
+
         if not isinstance(figure, matplotlib.figure.Figure):
             return
 
@@ -2325,11 +2350,16 @@ class GridCellDelegate(QStyledItemDelegate):
         :param index: Index of cell for which borders are drawn
 
         """
-
-        painter.setRenderHints(QPainter.LosslessImageRendering
-                               | QPainter.Antialiasing
-                               | QPainter.TextAntialiasing
-                               | QPainter.SmoothPixmapTransform)
+        try:
+            painter.setRenderHints(QPainter.LosslessImageRendering
+                                   | QPainter.Antialiasing
+                                   | QPainter.TextAntialiasing
+                                   | QPainter.SmoothPixmapTransform)
+        except AttributeError:
+            # Qt <5.13
+            painter.setRenderHints(QPainter.Antialiasing
+                                   | QPainter.TextAntialiasing
+                                   | QPainter.SmoothPixmapTransform)
 
         key = index.row(), index.column(), self.grid.table
         renderer = self.cell_attributes[key].renderer
