@@ -41,9 +41,12 @@ except ImportError:
 from typing import List, Tuple
 
 from PyQt5.QtCore import Qt, QModelIndex, QRectF, QPointF
-from PyQt5.QtGui import (QBrush, QColor, QPainter, QPalette, QPen,
+from PyQt5.QtGui import (QBrush, QPainter, QPalette, QPen,
                          QPainterPath, QPolygonF, QPainterPathStroker)
+
+from PyQt5.QtGui import QColor as __QColor
 from PyQt5.QtWidgets import QTableView, QStyleOptionViewItem
+from functools import lru_cache
 
 
 @contextmanager
@@ -111,6 +114,13 @@ def painter_rotate(painter: QPainter, rect: QRectF, angle: int = 0) -> QRectF:
         yield rect
 
 
+class QColor(__QColor):
+    """Hashable QColor"""
+
+    def __hash__(self):
+        return self.rgba()
+
+
 class GridCellNavigator:
     """Find neighbors of a cell"""
 
@@ -125,17 +135,20 @@ class GridCellNavigator:
         self.code_array = grid.model.code_array
         self.row, self.column, self.table = self.key = key
 
+        self.borderwidth_bottom_cache = grid.borderwidth_bottom_cache
+        self.borderwidth_right_cache = grid.borderwidth_right_cache
+
     @property
     def borderwidth_bottom(self) -> float:
         """Width of bottom border line"""
 
-        return self.code_array.cell_attributes[self.key].borderwidth_bottom
+        return self.borderwidth_bottom_cache[self.key]
 
     @property
     def borderwidth_right(self) -> float:
         """Width of right border line"""
 
-        return self.code_array.cell_attributes[self.key].borderwidth_right
+        return self.borderwidth_right_cache[self.key]
 
     @property
     def border_color_bottom(self) -> QColor:
@@ -155,7 +168,7 @@ class GridCellNavigator:
 
         return self.code_array.cell_attributes[self.key].merge_area
 
-    def _merging_key(self, key: Tuple[int, int, int]) -> Tuple[int, int, int]:
+    def merging_key(self, key: Tuple[int, int, int]) -> Tuple[int, int, int]:
         """Merging cell if cell is merged else cell key
 
         :param key: Key of cell that is checked for being merged
@@ -170,9 +183,9 @@ class GridCellNavigator:
 
         merge_area = self.merge_area
         if merge_area is None:
-            return [self._merging_key((self.row - 1, self.column, self.table))]
+            return [self.merging_key((self.row - 1, self.column, self.table))]
         _, left, _, right = merge_area
-        return [self._merging_key((self.row - 1, col, self.table))
+        return [self.merging_key((self.row - 1, col, self.table))
                 for col in range(left, right + 1)]
 
     def below_keys(self) -> List[Tuple[int, int, int]]:
@@ -180,9 +193,9 @@ class GridCellNavigator:
 
         merge_area = self.merge_area
         if merge_area is None:
-            return [self._merging_key((self.row + 1, self.column, self.table))]
+            return [self.merging_key((self.row + 1, self.column, self.table))]
         _, left, _, right = merge_area
-        return [self._merging_key((self.row + 1, col, self.table))
+        return [self.merging_key((self.row + 1, col, self.table))
                 for col in range(left, right + 1)]
 
     def left_keys(self) -> List[Tuple[int, int, int]]:
@@ -190,9 +203,9 @@ class GridCellNavigator:
 
         merge_area = self.merge_area
         if merge_area is None:
-            return [self._merging_key((self.row, self.column - 1, self.table))]
+            return [self.merging_key((self.row, self.column - 1, self.table))]
         top, _, bottom, _ = merge_area
-        return [self._merging_key((row, self.column - 1, self.table))
+        return [self.merging_key((row, self.column - 1, self.table))
                 for row in range(top, bottom + 1)]
 
     def right_keys(self) -> List[Tuple[int, int, int]]:
@@ -200,30 +213,30 @@ class GridCellNavigator:
 
         merge_area = self.merge_area
         if merge_area is None:
-            return [self._merging_key((self.row, self.column + 1, self.table))]
+            return [self.merging_key((self.row, self.column + 1, self.table))]
         top, _, bottom, _ = merge_area
-        return [self._merging_key((row, self.column + 1, self.table))
+        return [self.merging_key((row, self.column + 1, self.table))
                 for row in range(top, bottom + 1)]
 
     def above_left_key(self) -> Tuple[int, int, int]:
         """Key of neighboring cell above left of the key cell"""
 
-        return self._merging_key((self.row - 1, self.column - 1, self.table))
+        return self.merging_key((self.row - 1, self.column - 1, self.table))
 
     def above_right_key(self) -> Tuple[int, int, int]:
         """Key of neighboring cell above right of the key cell"""
 
-        return self._merging_key((self.row - 1, self.column + 1, self.table))
+        return self.merging_key((self.row - 1, self.column + 1, self.table))
 
     def below_left_key(self) -> Tuple[int, int, int]:
         """Key of neighboring cell below left of the key cell"""
 
-        return self._merging_key((self.row + 1, self.column - 1, self.table))
+        return self.merging_key((self.row + 1, self.column - 1, self.table))
 
     def below_right_key(self) -> Tuple[int, int, int]:
         """Key of neighboring cell below right of the key cell"""
 
-        return self._merging_key((self.row + 1, self.column + 1, self.table))
+        return self.merging_key((self.row + 1, self.column + 1, self.table))
 
 
 @dataclass
@@ -245,6 +258,8 @@ class EdgeBorders:
     top_y: float
     bottom_y: float
 
+    _color_cache = None
+
     @property
     def _border_widths(self) -> Tuple[float, float, float, float]:
         """Tuple of border widths in order left, right, top, bottom"""
@@ -275,21 +290,30 @@ class EdgeBorders:
     def color(self) -> QColor:
         """Edge color, i.e. darkest color of thickest edge border"""
 
+        if self._color_cache is not None:
+            return self._color_cache
+
         max_border_width = max(self.width, self.height)
         colors = []
         for width, color in zip(self._border_widths, self._border_colors):
             if width == max_border_width:
                 colors.append(color)
         colors.sort(key=lambda color: color.lightnessF())
+        color = colors[0]
 
-        return colors[0]
+        self._color_cache = color
+
+        return color
 
 
 class CellEdgeRenderer:
     """Paints cell edges"""
 
+    intersection_cache = {}
+
     def __init__(self, painter: QPainter, center: QPointF,
-                 borders: EdgeBorders, clip_path: QPainterPath, zoom: float):
+                 borders: EdgeBorders, rect: QRectF, clip_path: QPainterPath,
+                 zoom: float):
         """
 
         Borders are provided by EdgeBorders in order: left, right, top, bottom
@@ -297,7 +321,8 @@ class CellEdgeRenderer:
         :param painter: Painter with which edge is drawn
         :param center: Edge center
         :param borders: Border widths and colors
-        :param clip_path: Clip rectangle that is requuired for QtSVG clipping
+        :param rect: Rect of the clip_path
+        :param clip_path: Clip rectangle that is required for QtSVG clipping
         :param zoom: Current zoom level
 
         """
@@ -305,6 +330,7 @@ class CellEdgeRenderer:
         self.painter = painter
         self.center = center
         self.borders = borders
+        self.rect = rect
         self.clip_path = clip_path
         self.zoom = zoom
 
@@ -318,17 +344,26 @@ class CellEdgeRenderer:
         width = self.borders.width * self.zoom
         height = self.borders.height * self.zoom
 
-        rect = QRectF(x-width/2, y-height/2, width, height)
-
-        rect_path = QPainterPath()  # Required for clipping in SVG export
-        rect_path.addRect(rect)
+        cache_key = (self.rect.x(), self.rect.y(),
+                     self.rect.width(), self.rect.height(),
+                     x, y, width, height)
 
         color = self.borders.color
-
         self.painter.setPen(QPen(Qt.NoPen))
         self.painter.setBrush(QBrush(color))
 
-        self.painter.drawPath(self.clip_path.intersected(rect_path))
+        try:
+            intersection = self.intersection_cache[cache_key]
+        except KeyError:
+            rect = QRectF(x-width/2, y-height/2, width, height)
+            rect_path = QPainterPath()  # Required for clipping in SVG export
+            rect_path.addRect(rect)
+
+            intersection = self.clip_path.intersected(rect_path)
+
+            self.intersection_cache[cache_key] = intersection
+
+        self.painter.drawPath(intersection)
         self.painter.setPen(QPen(Qt.SolidLine))
 
 
@@ -341,13 +376,70 @@ class QColorCache(dict):
 
     def __missing__(self, key):
         if key is None:
-            self[key] = qcolor = self.grid.palette().color(QPalette.Mid)
+            qcolor = QColor(self.grid.palette().color(QPalette.Mid))
         else:
-            self[key] = qcolor = QColor(*key)
+            qcolor = QColor(*key)
+
+        self[key] = qcolor
 
         return qcolor
 
 
+class BorderWidthBottomCache(dict):
+    """BorderWidthBottom cache"""
+
+    def __init__(self, grid, *args, **kwargs):
+        self.grid = grid
+        self.cell_attributes = grid.model.code_array.cell_attributes
+
+        super().__init__(*args, **kwargs)
+
+    def __missing__(self, key):
+        borderwidth_bottom = self.cell_attributes[key].borderwidth_bottom
+        self[key] = borderwidth_bottom
+
+        return borderwidth_bottom
+
+
+class BorderWidthRightCache(BorderWidthBottomCache):
+    """BorderWidthRight cache"""
+
+    def __missing__(self, key):
+        borderwidth_right = self.cell_attributes[key].borderwidth_right
+        self[key] = borderwidth_right
+
+        return borderwidth_right
+
+
+class EdgeBordersCache(dict):
+    """Cache of all EdgeBorders objects"""
+
+    def __missing__(self, key):
+        self[key] = edge_border = EdgeBorders(*key)
+
+        return edge_border
+
+
+class BorderColorBottomCache(BorderWidthBottomCache):
+    """BorderColorBottomCache cache"""
+
+    def __missing__(self, key):
+        border_color_bottom = self.cell_attributes[key].border_color_bottom
+        self[key] = border_color_bottom
+
+        return border_color_bottom
+
+
+class BorderColorRightCache(BorderWidthBottomCache):
+    """BorderColorBottomCache cache"""
+
+    def __missing__(self, key):
+        border_color_right = self.cell_attributes[key].border_color_right
+        self[key] = border_color_right
+
+        return border_color_right
+
+
 class CellRenderer:
     """Paints cells
 
@@ -378,12 +470,22 @@ class CellRenderer:
         self.option = option
         self.index = index
 
+        try:
+            self.painter.setRenderHint(QPainter.LosslessImageRendering, True)
+        except AttributeError:
+            # Qt <5.13
+            pass
+
+        self.painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
+
         self.cell_attributes = grid.model.code_array.cell_attributes
         self.key = index.row(), index.column(), self.grid.table
 
         self.cell_nav = GridCellNavigator(grid, self.key)
 
-        self.qcolor_cache = self.grid.qcolor_cache
+        self.qcolor_cache = grid.qcolor_cache
+        self.borderwidth_bottom_cache = grid.borderwidth_bottom_cache
+        self.borderwidth_right_cache = grid.borderwidth_right_cache
 
     def inner_rect(self, rect: QRectF) -> QRectF:
         """Returns inner rect that is shrunk by border widths
@@ -397,9 +499,9 @@ class CellRenderer:
         above_keys = self.cell_nav.above_keys()
         left_keys = self.cell_nav.left_keys()
 
-        width_top = min(self.cell_attributes[above_key].borderwidth_bottom
+        width_top = min(self.borderwidth_bottom_cache[above_key]
                         for above_key in above_keys)
-        width_left = min(self.cell_attributes[left_key].borderwidth_right
+        width_left = min(self.borderwidth_right_cache[left_key]
                          for left_key in left_keys)
         width_bottom = self.cell_nav.borderwidth_bottom
         width_right = self.cell_nav.borderwidth_right
@@ -426,8 +528,10 @@ class CellRenderer:
             self.grid.delegate.paint_(self.painter, zrect, self.option,
                                       self.index)
 
-    def _get_border_pen(self, width, zoom):
-        """Gets zoomed border pen
+    @staticmethod
+    @lru_cache(maxsize=65536)
+    def _get_border_pen(width, zoom):
+        """Gets zoomed border pen, white, fully transparent
 
         :param width: Unzoomed line width
         :param zoom: Current zoom level of grid
@@ -439,6 +543,52 @@ class CellRenderer:
         return QPen(QColor(255, 255, 255, 0), zoomed_width,
                     Qt.SolidLine, Qt.FlatCap, Qt.MiterJoin)
 
+    def _draw_line(self, point1: QPointF, point2: QPointF, width: float,
+                   color: QColor, clip_path: QPainterPath):
+        """Draws line
+
+        Uses drawLine for screen painting and QPainterPathStroker for
+        svg painting.
+
+        :param point1: Start point of line
+        :param point2: End point of line
+        :param width: Line width
+        :param color: Line color
+        :param clip_path: Clip rectangle
+
+        """
+
+        zoom = self.grid.zoom
+        alpha = max(0, round(255 - 255 * width * zoom))
+
+        if width * zoom > 1.01:
+            self.painter.setRenderHint(QPainter.Antialiasing, True)
+
+        screen_painting = self.grid.main_window.print_area is None
+
+        if screen_painting:
+            # Rendering for the screen
+            pen = QPen(color)
+            pen.setWidthF(width * zoom)
+            self.painter.setPen(pen)
+            self.painter.drawLine(point1, point2)
+        else:
+            # Rendering for the printer
+            pen = self._get_border_pen(width, zoom)
+
+            line_polygon = QPolygonF((point1, point2))
+            line_path = QPainterPath()
+            line_path.addPolygon(line_polygon)
+
+            stroker = QPainterPathStroker(pen)
+            stroked_path = stroker.createStroke(line_path)
+
+            self.painter.setPen(QPen(QColor(255, 255, 255, alpha)))
+            self.painter.setBrush(QBrush(color))
+            self.painter.drawPath(clip_path.intersected(stroked_path))
+
+        self.painter.setRenderHint(QPainter.Antialiasing, False)
+
     def paint_bottom_border(self, rect: QRectF, clip_path: QPainterPath):
         """Paint bottom border of cell
 
@@ -450,25 +600,13 @@ class CellRenderer:
         if not self.cell_nav.borderwidth_bottom:
             return
 
-        line_color = self.qcolor_cache[self.cell_nav.border_color_bottom]
-
         point1 = QPointF(rect.x(), rect.y() + rect.height())
         point2 = QPointF(rect.x() + rect.width(), rect.y() + rect.height())
-        line_polygon = QPolygonF((point1, point2))
-        line_path = QPainterPath()
-        line_path.addPolygon(line_polygon)
 
-        pen = self._get_border_pen(self.cell_nav.borderwidth_bottom,
-                                   self.grid.zoom)
-        stroker = QPainterPathStroker(pen)
-        stroked_path = stroker.createStroke(line_path)
+        width = self.cell_nav.borderwidth_bottom
+        color = self.qcolor_cache[self.cell_nav.border_color_bottom]
 
-        alpha = max(0, round(255 - 255 * self.cell_nav.borderwidth_bottom *
-                             self.grid.zoom))
-
-        self.painter.setPen(QPen(QColor(255, 255, 255, alpha)))
-        self.painter.setBrush(QBrush(line_color))
-        self.painter.drawPath(clip_path.intersected(stroked_path))
+        self._draw_line(point1, point2, width, color, clip_path)
 
     def paint_right_border(self, rect: QRectF, clip_path: QPainterPath):
         """Paint right border of cell
@@ -481,25 +619,13 @@ class CellRenderer:
         if not self.cell_nav.borderwidth_right:
             return
 
-        line_color = self.qcolor_cache[self.cell_nav.border_color_right]
-
         point1 = QPointF(rect.x() + rect.width(), rect.y())
         point2 = QPointF(rect.x() + rect.width(), rect.y() + rect.height())
-        line_polygon = QPolygonF((point1, point2))
-        line_path = QPainterPath()
-        line_path.addPolygon(line_polygon)
 
-        pen = self._get_border_pen(self.cell_nav.borderwidth_right,
-                                   self.grid.zoom)
-        stroker = QPainterPathStroker(pen)
-        stroked_path = stroker.createStroke(line_path)
+        width = self.cell_nav.borderwidth_right
+        color = self.qcolor_cache[self.cell_nav.border_color_right]
 
-        alpha = max(0, round(255 - 255 * self.cell_nav.borderwidth_bottom *
-                             self.grid.zoom))
-
-        self.painter.setPen(QPen(QColor(255, 255, 255, alpha)))
-        self.painter.setBrush(QBrush(line_color))
-        self.painter.drawPath(clip_path.intersected(stroked_path))
+        self._draw_line(point1, point2, width, color, clip_path)
 
     def paint_above_borders(self, rect: QRectF, clip_path: QPainterPath):
         """Paint lower borders of all above cells
@@ -524,26 +650,13 @@ class CellRenderer:
             above_rect_width = sum(self.grid.columnWidth(column)
                                    for column in columns)
 
-            line_color = self.qcolor_cache[above_cell_nav.border_color_bottom]
-
             point1 = QPointF(above_rect_x, rect.y())
             point2 = QPointF(above_rect_x + above_rect_width, rect.y())
-            line_polygon = QPolygonF((point1, point2))
-            line_path = QPainterPath()
-            line_path.addPolygon(line_polygon)
 
-            pen = self._get_border_pen(above_cell_nav.borderwidth_bottom,
-                                       self.grid.zoom)
-            stroker = QPainterPathStroker(pen)
-            stroked_path = stroker.createStroke(line_path)
+            width = above_cell_nav.borderwidth_bottom
+            color = self.qcolor_cache[above_cell_nav.border_color_bottom]
 
-            alpha = max(0, round(255 - 255 *
-                                 above_cell_nav.borderwidth_bottom *
-                                 self.grid.zoom))
-
-            self.painter.setPen(QPen(QColor(255, 255, 255, alpha)))
-            self.painter.setBrush(QBrush(line_color))
-            self.painter.drawPath(clip_path.intersected(stroked_path))
+            self._draw_line(point1, point2, width, color, clip_path)
 
     def paint_left_borders(self, rect: QRectF, clip_path: QPainterPath):
         """Paint right borders of all left cells
@@ -567,26 +680,13 @@ class CellRenderer:
             left_rect_y = self.grid.rowViewportPosition(rows[0])
             left_rect_height = sum(self.grid.rowHeight(row) for row in rows)
 
-            line_color = self.qcolor_cache[left_cell_nav.border_color_right]
-
             point1 = QPointF(rect.x(), left_rect_y)
             point2 = QPointF(rect.x(), left_rect_y + left_rect_height)
-            line_polygon = QPolygonF((point1, point2))
-            line_path = QPainterPath()
-            line_path.addPolygon(line_polygon)
 
-            pen = self._get_border_pen(left_cell_nav.borderwidth_right,
-                                       self.grid.zoom)
-            stroker = QPainterPathStroker(pen)
-            stroked_path = stroker.createStroke(line_path)
+            width = left_cell_nav.borderwidth_right
+            color = self.qcolor_cache[left_cell_nav.border_color_right]
 
-            alpha = max(0, round(255 - 255 *
-                                 left_cell_nav.borderwidth_right *
-                                 self.grid.zoom))
-
-            self.painter.setPen(QPen(QColor(255, 255, 255, alpha)))
-            self.painter.setBrush(QBrush(line_color))
-            self.painter.drawPath(clip_path.intersected(stroked_path))
+            self._draw_line(point1, point2, width, color, clip_path)
 
     def paint_top_left_edge(self, rect: QRectF, clip_path: QPainterPath):
         """Paints top left edge of the cell
@@ -627,12 +727,14 @@ class CellRenderer:
         top_y = rect.y() - rect.height()
         bottom_y = rect.y()
 
-        borders = EdgeBorders(left_width, right_width, top_width, bottom_width,
-                              left_color, right_color, top_color, bottom_color,
-                              left_x, right_x, top_y, bottom_y)
+        key = (left_width, right_width, top_width, bottom_width,
+               left_color, right_color, top_color, bottom_color,
+               left_x, right_x, top_y, bottom_y)
 
-        renderer = CellEdgeRenderer(self.painter, center, borders, clip_path,
-                                    self.grid.zoom)
+        borders = self.grid.edge_borders_cache[key]
+
+        renderer = CellEdgeRenderer(self.painter, center, borders, rect,
+                                    clip_path, self.grid.zoom)
         renderer.paint()
 
     def paint_top_right_edge(self, rect: QRectF, clip_path: QPainterPath):
@@ -672,12 +774,14 @@ class CellRenderer:
         top_y = rect.y() - rect.height()
         bottom_y = rect.y()
 
-        borders = EdgeBorders(left_width, right_width, top_width, bottom_width,
-                              left_color, right_color, top_color, bottom_color,
-                              left_x, right_x, top_y, bottom_y)
+        key = (left_width, right_width, top_width, bottom_width,
+               left_color, right_color, top_color, bottom_color,
+               left_x, right_x, top_y, bottom_y)
 
-        renderer = CellEdgeRenderer(self.painter, center, borders, clip_path,
-                                    self.grid.zoom)
+        borders = self.grid.edge_borders_cache[key]
+
+        renderer = CellEdgeRenderer(self.painter, center, borders, rect,
+                                    clip_path, self.grid.zoom)
         renderer.paint()
 
     def paint_bottom_left_edge(self, rect: QRectF, clip_path: QPainterPath):
@@ -718,12 +822,14 @@ class CellRenderer:
         top_y = rect.y() + rect.height()
         bottom_y = rect.y() + 2 * rect.height()
 
-        borders = EdgeBorders(left_width, right_width, top_width, bottom_width,
-                              left_color, right_color, top_color, bottom_color,
-                              left_x, right_x, top_y, bottom_y)
+        key = (left_width, right_width, top_width, bottom_width,
+               left_color, right_color, top_color, bottom_color,
+               left_x, right_x, top_y, bottom_y)
 
-        renderer = CellEdgeRenderer(self.painter, center, borders, clip_path,
-                                    self.grid.zoom)
+        borders = self.grid.edge_borders_cache[key]
+
+        renderer = CellEdgeRenderer(self.painter, center, borders, rect,
+                                    clip_path, self.grid.zoom)
         renderer.paint()
 
     def paint_bottom_right_edge(self, rect: QRectF, clip_path: QPainterPath):
@@ -763,12 +869,14 @@ class CellRenderer:
         top_y = rect.y() + rect.height()
         bottom_y = rect.y() + 2 * rect.height()
 
-        borders = EdgeBorders(left_width, right_width, top_width, bottom_width,
-                              left_color, right_color, top_color, bottom_color,
-                              left_x, right_x, top_y, bottom_y)
+        key = (left_width, right_width, top_width, bottom_width,
+               left_color, right_color, top_color, bottom_color,
+               left_x, right_x, top_y, bottom_y)
 
-        renderer = CellEdgeRenderer(self.painter, center, borders, clip_path,
-                                    self.grid.zoom)
+        borders = self.grid.edge_borders_cache[key]
+
+        renderer = CellEdgeRenderer(self.painter, center, borders, rect,
+                                    clip_path, self.grid.zoom)
         renderer.paint()
 
     def paint_borders(self, rect):
