@@ -38,6 +38,7 @@
 from functools import partial
 from pathlib import Path
 
+from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (
         QMenuBar, QMenu, QAction, QMainWindow, QWidget, QToolBar)
 
@@ -352,6 +353,8 @@ class BorderChoiceMenu(QMenu):
 
         """
 
+        self.actions = actions
+
         super().__init__()
 
         self.setTitle("Formatted borders")
@@ -366,6 +369,14 @@ class BorderChoiceMenu(QMenu):
         self.addAction(actions.format_borders_inner)
         self.addAction(actions.format_borders_top_bottom)
 
+        self.triggered.connect(self.on_selection)
+
+    def on_selection(self, event):
+        """Event handler"""
+
+        self.actions.parent.format_toolbar.border_menu_button.setAttribute(
+            Qt.WA_UnderMouse, False)
+
 
 class BorderWidthMenu(QMenu):
     """QMenu for choosing the cell border width"""
@@ -376,6 +387,8 @@ class BorderWidthMenu(QMenu):
 
         """
 
+        self.actions = actions
+
         super().__init__()
 
         self.setTitle("Border width")
@@ -390,6 +403,14 @@ class BorderWidthMenu(QMenu):
         self.addAction(actions.format_borders_32)
         self.addAction(actions.format_borders_64)
 
+        self.triggered.connect(self.on_selection)
+
+    def on_selection(self, event):
+        """Event handler"""
+
+        self.actions.parent.format_toolbar.border_menu_button.setAttribute(
+            Qt.WA_UnderMouse, False)
+
 
 class GridContextMenu(QMenu):
     """Context menu for grid"""
