@@ -796,7 +796,7 @@ class DataArray:
 
         for i, key_ele in enumerate(key):
 
-            # Get first element of key that is a slice
+            # Recursively replace first element of key that is a slice
             if isinstance(key_ele, slice):
                 slc_keys = range(*key_ele.indices(self.dict_grid.shape[i]))
                 key_list = list(key)
@@ -1221,9 +1221,6 @@ class CodeArray(DataArray):
     # Cache for frozen objects
     frozen_cache = {}
 
-    # Custom font storage
-    custom_fonts = {}
-
     # Safe mode: If True then Whether pyspread is operating in safe_mode
     # In safe_mode, cells are not evaluated but its code is returned instead.
     safe_mode = False
@@ -1263,6 +1260,16 @@ class CodeArray(DataArray):
 
         """
 
+        code = self(key)
+
+        if code is None:
+            return
+
+        # Cached cell handling
+
+        if repr(key) in self.result_cache:
+            return self.result_cache[repr(key)]
+
         if not any(isinstance(k, slice) for k in key):
             # Button cell handling
             if self.cell_attributes[key].button_cell is not False:
@@ -1274,20 +1281,16 @@ class CodeArray(DataArray):
                     return self.frozen_cache[repr(key)]
                 # Frozen cache is empty.
                 # Maybe we have a reload without the frozen cache
-                result = self._eval_cell(key, self(key))
+                result = self._eval_cell(key, code)
                 self.frozen_cache[repr(key)] = result
                 return result
 
         # Normal cell handling
 
-        if repr(key) in self.result_cache:
-            return self.result_cache[repr(key)]
+        result = self._eval_cell(key, code)
+        self.result_cache[repr(key)] = result
 
-        elif self(key) is not None:
-            result = self._eval_cell(key, self(key))
-            self.result_cache[repr(key)] = result
-
-            return result
+        return result
 
     def _make_nested_list(self, gen: Union[Iterable, Iterable[Iterable],
                                            Iterable[Iterable[Iterable]]]
@@ -1327,7 +1330,7 @@ class CodeArray(DataArray):
 
     def exec_then_eval(self, code: str,
                        _globals: dict = None, _locals: dict = None):
-        """execs multuiline code and returns eval of last code line
+        """execs multiline code and returns eval of last code line
 
         :param code: Code to be executed / evaled
         :param _globals: Globals dict for code execution and eval
